//Module Imports
const { resolve: resolvePath } = require("path");
const {
  existsSync: fexists,
  mkdirSync: mkdir,
  readdirSync: readdir,
  unlinkSync: fremove,
} = require("fs");
const config = require("../config.json");
const mongoose = require("mongoose");
mongoose.connect(`${config.Storage.NubianDatabase}/nubian?authSource=admin`, {
  useUnifiedTopology: true,
  useNewUrlParser: true,
  user: "admin",
  pass: process.env.NUBIAN_DATABASE_PASSWORD,
});

const users = require("../schemas/user");
const files = require("../schemas/file");

const zipDir = resolvePath(config.Storage.ZipPath);

function authorizedToView(userId, file) {
  if (`${file.owner}` == (userId = `${userId}`) || file.public) return true;
  if (file.view.includes(userId) || file.edit.includes(userId)) return true;
  return false;
}

function authorizedToEdit(userId, file) {
  return `${file.owner}` == `${userId}` || file.edit.includes(`${userId}`);
}

function getFile(userId, fileId) {
  return files.findOne({ _id: fileId }).then((file) => {
    if (authorizedToView(userId, file)) return file;
    return null;
  });
}

function deleteFiles(userId, fileIds) {
  return files.find({ _id: { $in: fileIds } }).then((databaseFiles) => {
    var failed = [];
    filesByOwner = {};
    databaseFiles.forEach((file) => {
      if (file.owner in filesByOwner) filesByOwner[file.owner].push(file);
      else filesByOwner[file.owner] = [file];
    });
    for (var owner in filesByOwner) {
      var deleteSize = 0;
      for (var i = filesByOwner[owner].length - 1; i >= 0; i--) {
        let file = filesByOwner[owner][i];
        if (!authorizedToEdit(userId, file)) {
          failed.push(`${file._id}`);
          filesByOwner[owner].splice(i, 1);
        } else deleteSize += file.size;
      }
      users
        .updateOne(
          { _id: owner, usedStorage: { $gte: deleteSize } },
          {
            $pull: { owned: { $in: filesByOwner[owner] } },
            $inc: {
              usedStorage: -deleteSize,
            },
          }
        )
        .exec();
      fileIds = fileIds.filter((fileId) => !failed.includes(fileId));
      rfiles = databaseFiles.filter((file) => fileIds.includes(`${file._id}`));
      files
        .deleteMany({
          _id: { $in: fileIds },
        })
        .exec();
      return { files: rfiles, failed };
    }
  });
}

function publicfyFiles(userId, targetFiles) {
  return files.find({ _id: { $in: targetFiles } }).then((databaseFiles) => {
    var failed = [];
    databaseFiles.forEach((file) => {
      if (!authorizedToEdit(userId, file)) failed.push(`${file._id}`);
      else files.updateOne({ _id: file._id }, { public: !file.public }).exec();
    });
    return failed;
  });
}

function createUser(cairoUuid) {
  return users.create(
    {
      cairoUuid,
      usedStorage: 0,
      storage: config.Storage.UserStorageSize * config.Storage.UserStorageUnit,
      owned: [],
      shared: [],
    },
    (err, result) => {
      if (err) return err;
      return result;
    }
  );
}

function getUserByCairoUuid(cairoUuid) {
  return users.findOne({ cairoUuid: cairoUuid }, (err, result) => {
    if (result == null) createUser(cairoUuid);
    if (err) console.error(err);
  });
}

function getUserById(userId) {
  return users.findOne({ _id: userId }, (err, result) => {
    if (result == null) createUser(cairoUuid);
    if (err) console.error(err);
  });
}

function uploadFile(userId, fileData) {
  return getUserById(userId).then((user) => {
    if (user.usedStorage + fileData.size > user.storage) return null;
    return users
      .updateOne({ _id: userId }, { $inc: { usedStorage: fileData.size } })
      .then(() => createFile(user._id, fileData))
      .then((file) => {
        if (file == null) return null;
        users.updateOne({ _id: userId }, { $push: { owned: file._id } }).then();
        return file;
      });
  });
}

function createFile(userId, fileData) {
  return files.create({
    path: fileData.path,
    owner: userId,
    name: fileData.originalname,
    date: fileData.filename.substring(0, fileData.filename.indexOf("-")),
    size: fileData.size,
    public: false,
    edit: [],
    view: [],
  });
}

function cleanZips() {
  console.log("Would clean zips");
  return;
  var zipUuid;
  const time = Date.now();
  readdir(zipDir).forEach((file) => {
    zipUuid = file.substring(0, file.indexOf(".zip"));
    fileStorage.updateEntry(zipUuid, "zips", (entry, deleteEntry) => {
      if (entry == null) return;
      if (entry.building === true) return;
      if (entry.exp <= time) {
        deleteEntry();
        fremove(entry.path);
      }
    });
  });
}
module.exports = {
  deleteFiles,
  getFile,
  createUser,
  getUserById,
  getUserByCairoUuid,
  publicfyFiles,
  uploadFile,
  cleanZips,
};
